## explore_butterflies.R
##    R script that reproduces our exploration analysis of the Roland and Matter
##    apline butterfly data.


# Load packages -----------------------------------------------------------

# devtools::install_github('atredennick\modselr')  # for the butterfly data
library(modselr)    # in-house package that includes data
library(tidyverse)  # data wrangling functions
library(lubridate)  # date and time functions
library(broom)      # tidy summaries of model fits
library(ggplot2)    # plotting
library(ggthemes)   # pleasing ggplot2 themes
library(cowplot)    # functions for combining ggplot2 objects
library(MASS)       # stepAIC function
library(lme4)       # mixed-effects model fittin

theme_set(theme_minimal())  # set ggplot2 theme globally


# Reformat the data -------------------------------------------------------

butterfly_long <- butterfly %>%
  filter(year < 2015) %>%  # drop 2015, incomplete data
  gather(covariate, value, -year, -Rt, -meada) %>%
  drop_na()  # there are many NAs for some year*covariate combos
             # remove here to avoid NA correlations, etc.

# Calculate correlations --------------------------------------------------

cov_cors <- butterfly_long %>%
  group_by(covariate) %>%
  summarise(rho = abs(cor(value, Rt))) %>%
  filter(rho > 0.3) %>%
  pull(covariate)

# Plot the relationships --------------------------------------------------

butterfly_long %>%
  filter(covariate %in% cov_cors) %>%
  ggplot(aes(x = value, y = Rt, group = meada))+
    geom_point(size = 1, shape = 1, alpha = 0.4)+
    geom_smooth(method = "lm", se = FALSE, size = 0.5, color = "black")+
    facet_wrap(~covariate, scales = "free", nrow = 3)+
    labs(x = "Covariate value", y = expression(R[t]))+
    theme(axis.text = element_text(size = 6))  -> outplot

saveRDS(outplot, file="results/clim_corrs_fig.RDS")

# GLMM stepwise -----------------------------------------------------------

# Fit a full model and do drop-one selection
corr_covar_df <- butterfly_long %>%
  filter(covariate %in% cov_cors) %>%
  spread(covariate, value) 

covars <- paste(colnames(corr_covar_df)[-c(1:3)], collapse = "+")
my_formula <- paste("Rt ~", covars, "+ (1|meada)")
full_model <- lmer(as.formula(my_formula), data = corr_covar_df)

drop_results <- drop1(full_model, test = "Chisq") %>%
  rownames_to_column("dropped_covariate") %>%
  rename(p_value = `Pr(Chi)`) %>%
  mutate(
    include = ifelse(p_value < 0.05, "yes", "no")
  ) %>%
  as_tibble()

# multiple comparison corrections
p_adjusted_BH <- p.adjust(drop_results$p_value,method="BH",n=96)
include_BH <- ifelse(p_adjusted_BH < 0.05, "yes", "no")
p_adjusted_holm <- p.adjust(drop_results$p_value,method="holm",n=96)
include_holm <- ifelse(p_adjusted_holm < 0.05, "yes", "no")
drop_results <- cbind(drop_results,p_adjusted_BH,include_BH,p_adjusted_holm,include_holm)

# Fit final model based on drop-one results
covars_not_dropped <- drop_results %>%
  filter(include == "yes") %>%
  pull(dropped_covariate)

final_covar_df <- butterfly_long %>%
  filter(covariate %in% covars_not_dropped) %>%
  spread(covariate, value) 
  
final_formula <- paste(
  "Rt ~", 
  paste(colnames(final_covar_df)[-c(1:3)], 
        collapse = "+"), 
  "+ (1|meada)"
)

final_model <- lmer(as.formula(final_formula), data = final_covar_df) %>%
  tidy() %>%
  filter(group == "fixed") %>%
  dplyr::select(-group) %>%
  as_tibble()

final_model <- merge(final_model,drop_results[,c("dropped_covariate", "p_value", "include",           
                      "p_adjusted_BH","include_BH","p_adjusted_holm","include_holm")],
                     by.x="term",by.y="dropped_covariate",all.x=T)

final_model_fname <- "./results/butterfly_drop1_results.csv"
write_csv( x = final_model, path = final_model_fname)

# Perform climate window analysis -----------------------------------------

# reformat climate data, adding dates
month_nums <- tibble(month=tolower(month.abb))
month_nums$num <- 1:12

covars_dated <- butterfly %>%
  dplyr::filter(meada == "F") %>% # all meadows have same climate data, just use one
  dplyr::select(-Nt, -logNt,-Rt, -meada) %>%  # drop non-climate vars
  dplyr::select(-maxsnow, -Datesnowmelt, -aprraintmn1, 
         -mayraintmn1, -junraintmn1, -julraintmn1, 
         -augraintmn1, -sumraint, -sumraintmn1) %>%  # drop non-dated variables
  gather(covariate, value, -year) %>%
  mutate(
    month = substr(covariate, 1, 3),
    covariate = substr(covariate, 4, nchar(covariate))
  ) %>%
  left_join(month_nums, by = "month") %>%
  mutate(date = ymd(paste0(as.character(year),month,"-01"))) %>%
  dplyr::select(-year, -month, -num) %>%
  filter(year(date) < 2014) %>%
  spread(covariate, value)

# add dates to response variables, assume obs always on July 1
response_dated <- butterfly %>%
  dplyr::select(meada, year, Nt,logNt,Rt) %>%
  filter(year < 2014 & year > 1996) %>% # avoid missing climate values
  mutate(date = ymd(paste0(as.character(year),"06","-01")))

# run climwin for max temperature
library(climwin)
climwin_fit <- slidingwin(xvar = list(mean = covars_dated$mean),
                   cdate = covars_dated$date,
                   bdate = response_dated$date,
                   baseline = lmer(Rt ~ logNt + (1|meada), data = response_dated, REML=F),
                   range = c(12, 0),
                   type = "relative", stat = "mean",
                   func = c("lin"), cmissing = "method1", cinterval = "month")

# save output
saveRDS(climwin_fit,file="./results/climwin.RDS")





