## understand_butterflies.R
##    R script that reproduces our "understanding" analysis of the Roland and 
##    Matter alpine butterfly data.

# Load packages -----------------------------------------------------------

library(modselr)
library(tidyverse)
library(ggthemes)
library(lme4)
library(xtable)


# Reformat the data -------------------------------------------------------

# pull next year's log population size out of the Rt variable
butterfly$logNtnext <- butterfly$logNt + butterfly$Rt

butterfly_long <- butterfly %>%
  filter(year < 2014) %>%
  gather(covariate, value, -year, -Rt, -meada, -logNtnext) 


# Generate and extract relevant covariates -------------------------------

my_variables <- c("logNt", 
                  "novextmax", 
                  "decextmax",
                  "novsnowfall",
                  "decsnowfall")

model_data <- butterfly_long %>%
  filter(covariate %in% my_variables) %>%
  spread(covariate, value)

temperature_data <- model_data %>%
  dplyr::select(year, meada, ends_with("extmax")) %>%
  gather(covariate, value, -year, -meada) %>%
  group_by(year, meada) %>%
  summarise(winter_mean_extmax = mean(value))

snow_data <- model_data %>%
  dplyr::select(year, meada, ends_with("snowfall")) %>%
  gather(covariate, value, -year, -meada) %>%
  group_by(year, meada) %>%
  summarise(winter_mean_snow = mean(value))

test_data <- model_data %>%
  dplyr::select(year, meada, logNtnext, logNt) %>%
  left_join(snow_data, by = c("year", "meada")) %>%
  left_join(temperature_data, by = c("year", "meada")) %>%
  mutate(
    meada = as.factor(meada)
  )

# Just to see if we believe the Normality assumptions
all_mod <- lm(logNtnext ~ logNt + winter_mean_snow*winter_mean_extmax + meada,
              data = test_data)

plot(all_mod)  # not bad. Residuals have a bit of kurtosis (3.23), otherwise good. 
 

#  Likelihood ratio version with lmer ------------------------------------------------

interaction_mod <- lmer(logNtnext ~ logNt + winter_mean_snow*winter_mean_extmax + (1|meada),
                        data=test_data)

main_effect_mod <- lmer(logNtnext ~ logNt + winter_mean_snow + winter_mean_extmax + (1|meada),
                        data=test_data)
 
print(anova(main_effect_mod,interaction_mod))  # test for snow interaction


# Save model coefficients -------------------------------------------------

saveRDS(interaction_mod, file = "./results/understanding_interaction_model.RDS")


# Make profile plot "data" ----------------------------------------------
# See https://biologyforfun.wordpress.com/2015/06/17/confidence-intervals-for-prediction-in-glmms/
# for methods, but pretty self explanatory

winter_temp_vec <- seq(min(test_data$winter_mean_extmax),max(test_data$winter_mean_extmax))

upper_snow <- quantile(test_data$winter_mean_snow, 0.9) 
lower_snow <- quantile(test_data$winter_mean_snow, 0.1)

newdat <- tibble(winter_mean_extmax = winter_temp_vec,
               meada = test_data$meada[1],
               logNt = mean(test_data$logNt),
               winter_mean_snow = lower_snow,
               snow_level = "low") %>%
  bind_rows(
    tibble(winter_mean_extmax = winter_temp_vec,
           meada = test_data$meada[1],
           logNt = mean(test_data$logNt),
           winter_mean_snow = upper_snow,
           snow_level = "high")
  ) %>%
  as.data.frame()

m <- interaction_mod
allout <- tibble()  # empty object
for(do_level in unique(newdat$snow_level)) {
  tmpdat <- newdat %>%
    filter(snow_level == do_level)
  
  mm <- model.matrix(~ logNt + winter_mean_snow*winter_mean_extmax, tmpdat)
  tmpdat$y <- mm%*%fixef(m) 
  # predict(m, newdat, re.form=NA) would give the same results
  pvar1 <- diag(mm %*% tcrossprod(vcov(m),mm))
  
  outdat <- data.frame(tmpdat,
                       lower_ci = tmpdat$y-1.96*sqrt(pvar1),
                       upper_ci = tmpdat$y+1.96*sqrt(pvar1))
  allout <- bind_rows(allout, outdat)
}

saveRDS(allout, file = "results/profile_plot_data.RDS")
