# R script for the prediction analysis in the model selection guide paper


# Load packages -----------------------------------------------------------

library(modselr)    # in-house package with data
library(tidyverse)  # all the data wrangling tools
library(glmnet)     # functions for statistical regularization
library(RColorBrewer)

# Reformat the data 

# pull next year's log population size out of the Rt variable
butterfly$logNtnext <- butterfly$logNt + butterfly$Rt

butterfly_to_predict <- butterfly %>%
  filter(year != 2015) %>%  # remove newest year w/out data
  select_if(~ !any(is.na(.))) %>%  # remove columns with NA
  gather(covariate, value, -year, -Rt, -meada,-logNtnext)  %>%  # make it tidy
  filter(covariate != "Nt")  # remove raw abundance covariate

# Reduce covariates for consideration 
keep_covars <- c("decextmax", "decextmin", "logNt", "marmeanmax", "maymean",
                 "maymeanmin", "novextmax", "novmeanmax", "octmeanmin")
butterfly_to_predict <- butterfly_to_predict %>%
  filter(covariate %in% keep_covars)

# put in wide format
butterfly_to_predict <- spread(butterfly_to_predict, covariate, value)

# set up covariates
X <- butterfly_to_predict %>%
  dplyr::select(-year, -Rt, -meada, -logNtnext)
X <- as.matrix(X)
X <- scale(X)  # standardize the continuous covariates

# add meadow dummy variables
dummies <- model.matrix(~meada,butterfly_to_predict)
dummies <- dummies[,-1] # drop intercept column, glmnet will include it
X <- cbind(dummies,X)
rm(dummies)

# try each separate year as a test set
obs_pred <- data.frame(year = numeric(),obs = numeric(),
                       pred_opt = numeric(), pred_null = numeric(),
                       pred_nopenalty = numeric())

for(i_yr in sort(unique(butterfly_to_predict$year))){
  
  # split into training and test sets
  test_year <- i_yr
  
  train <- which(butterfly_to_predict$year!=test_year)
  test <- which(butterfly_to_predict$year==test_year)
  
  X_train <- X[train,]
  X_test <- X[test,]
  
  y_train <- butterfly_to_predict$logNtnext[train]
  y_test <- butterfly_to_predict$logNtnext[test]
  
  train_yrs <- butterfly_to_predict$year[train]
  
  # Fit model -----------------------------------------------
  
  # penalty values
  lambdas <- 10^seq(0 ,-7, length.out = 200)  # sequence of penalties to test
  
  # don't penalize previous year's abundance or meadow ID
  pen_facts <- rep(1, times = ncol(X))
  logNtId <- which(colnames(X) == "logNt")
  meadaId <- grep("meada",colnames(X))
  pen_facts[c(logNtId,meadaId)] <- 0  
  
  # specify leave-one-year-out cross-validation
  my_folds <- as.numeric(as.factor(train_yrs))
  
  # Fit the model at all levels of lambda
  fit <- cv.glmnet(
    x = X_train, 
    y = y_train, 
    family = "gaussian",
    alpha = 0,  # 0 == ridge regression, 1 == lasso, 0.5 ~~ elastic net
    #lambda = lambdas, 
    type.measure="mse",
    penalty.factor = pen_facts,
    foldid = my_folds,
    standardize = FALSE
  )
  
  # look at CV score vs penalty plot
  plot(log(fit$lambda),fit$cvm,main=test_year)
  
  # ridge model predictions
  optimal_pred <- predict(fit,newx=X_test,s="lambda.min")
  
  # null model and predictions
  X_null_train <- data.frame(X_train[,c(1:10,13)])
  X_null_test <- data.frame(X_test[,c(1:10,13)])
  null_fit <- lm(y_train ~ ., data = X_null_train)
  null_pred <- predict(null_fit, newdata = X_null_test)

  # no penalty model and predictions
  X_train_df <- data.frame(X_train)
  nopen <- lm(y_train ~ ., data = X_train_df)
  nopen_pred <- predict(nopen,newdata=data.frame(X_test))
  
  # save data
  tmp <- data.frame(year = rep(test_year,11),obs=y_test,
                    pred_opt=optimal_pred[,1], pred_null=null_pred,
                    pred_nopenalty=nopen_pred)
  obs_pred <- rbind(obs_pred,tmp)
  
  # save one example of model fits
  if(i_yr==1995){
    saveRDS(fit,file="results/glmnet_fit.RDS")
    saveRDS(nopen,file="results/nopenalty_fit.RDS")
  }
}

# calculate correlations
my_cors <- c(cor(obs_pred$pred_opt,obs_pred$obs),
            cor(obs_pred$pred_null,obs_pred$obs), 
            cor(obs_pred$pred_nopenalty,obs_pred$obs))

# calculate mse
my_mse <- c(mean((obs_pred$pred_opt - obs_pred$obs)^2) ,
            mean((obs_pred$pred_null - obs_pred$obs)^2),
            mean((obs_pred$pred_nopenalty - obs_pred$obs)^2))

# print results
names(my_cors) <- names(my_mse) <- c("optimal","null","no penalty")
print(my_cors)
print(my_mse)

# save obs vs pred results
write.csv(obs_pred,"results/obs_vs_pred.csv",row.names=F)




