# R script to make figures and tables for the paper.

library(ggplot2)
library(ggforce)
library(xtable)
library(tidyverse)
library(climwin)

# Venn diagram figure -----------------------------------------------
myplot <- tibble(x = c(0, 2.35, -2.35),
                 y = c(1, 1, 1),
                 labels = c("Prediction", "Inference", "Exploration")) %>%
  ggplot(aes(x0 = x, y0 = y, r = 1.5)) +
  geom_circle(alpha = 0.3, size = 1, colour = 'grey', fill = "grey") +
  geom_text(aes(x = x, y = y, label = labels), size = 2.8) +
  geom_curve(x = -2.35, y = -0.5, xend = 2.35, yend = -0.5,
             arrow = arrow(length = unit(0.03, "npc")), size = 0.2) +
  geom_text(aes(x = x[1], y = y[1]-2.3, label = "Independent studies"), 
            size = 2.8, check_overlap = TRUE) +
  scale_y_continuous(limits = c(-3,3)) +
  coord_fixed() +
  theme_void()

pdf('figures/venn_diagram.pdf',height = 3, width = 5)
print(myplot)
dev.off()

# Butterfly data figure --------------------------------------------------

alpha_level <- 0.6  # transparency level
line_size <- 0.4  # line width

butterfly %>%
  ggplot(aes(x = year, y = Nt, group = meada)) +
  geom_line(alpha = alpha_level, size = line_size) +
  guides(linetype = FALSE, shape = FALSE)+
  labs(x = "Year", y = expression(italic(N))) +
  scale_x_continuous(limits = c(1995,2015)) +
  theme_minimal() -> nt_plot

butterfly %>%
  dplyr::filter(year < 2015) %>%  # no pop growth in 2015
  ggplot(aes(x = year, y = Rt, group = meada)) +
  geom_line(alpha = alpha_level, size = line_size) +
  guides(linetype = FALSE, shape = FALSE) +
  labs(x = "Year", y = expression(italic(R))) +
  scale_x_continuous(limits = c(1995,2015)) +
  theme_minimal() -> rt_plot

myplot <- plot_grid(nt_plot, rt_plot, align = "v", labels = "AUTO", ncol = 1)

pdf('figures/butterfly_time_series.pdf' , width = 2.5, height = 4.5)
print(myplot)
dev.off()

# Modeling for exploration ###############################################

# Explore correlations figure--------------------------------------------

myplot <- readRDS("results/clim_corrs_fig.RDS")
pdf('figures/correlations.pdf',height = 6, width = 7.5 )
print(myplot)
dev.off()

## Drop1-table---------------------------------------------------------

mytable <- read_csv("results/butterfly_drop1_results.csv") %>%
  dplyr::rename(
    Covariate = term,
    Estimate = estimate,
    SE = std.error,
    `t statistic` = statistic,
    `Drop1 p value` = p_value,
    include = include,
    `adjusted p value (BH)` = p_adjusted_BH,
    `include (BH)` = include_BH,
    `adjusted p value (holm)` = p_adjusted_holm,
    `include (holm)` = include_holm
  ) 

print(xtable(mytable,caption="Drop1 results"),caption.placement="top",
      type="latex",file="results/butterfly_drop1_table.tex")

# Climwin figure -------------------------------------------------

climwin_fit <- readRDS("./results/climwin.RDS")

# for some reason, this plot to pdf fails when the script
# is sourced, but runs when just the next three lines are executed
pdf("./figures/climwin.pdf",height=4,width=4)
plotdelta(climwin_fit[[1]]$Dataset)
dev.off()


rm(climwin_fit)

## Modeling for Inference ##############################################

# Read in model summary and make table
interaction_mod <- readRDS("./results/understanding_interaction_model.RDS")

print(xtable(coef(summary(interaction_mod)),caption="Inference, fixed effects"),
      caption.placement="top",
      file="./results/understanding_model_results.tex")

# Make profile plot
allOut <- readRDS(file = "./results/profile_plot_data.RDS")

ggplot(allout, aes(x = winter_mean_extmax, y = y,
                   color = snow_level, fill = snow_level)) +
  geom_hline(aes(yintercept = 0), linetype = 3) +
  geom_ribbon(aes(ymin = lower_ci, ymax = upper_ci), color = NA, alpha = 0.25) +
  geom_line() +
  scale_color_manual(values = rev(c("coral","dodgerblue")), 
                     labels = c("High snow", "Low snow"),
                     name = NULL) +
  scale_fill_manual(values = rev(c("coral","dodgerblue")),
                    labels = c("High snow", "Low snow"),
                    name = NULL) +
  scale_y_continuous(breaks = c(0,1,2)) +
  scale_x_continuous(breaks = seq(2, 8, by = 2)) +
  labs(x = "Mean maximum temperature", y = "(log) Population size") +
  theme_few() +
  theme(legend.position = c(0.8, 0.85))

ggsave("./figures/inference_profile_CIs.pdf", height=3.2, width=4, units = "in")


# Modeling for prediction ########################################

# regularization figure, show shrinkage ----------------------------------

ridge_fit <- readRDS("results/glmnet_fit.RDS")
nopenalty_fit <- readRDS("results/nopenalty_fit.RDS")

# extract and compare climate coefs
coef_names <- names(coef(nopenalty_fit))
cut_names <- c("(Intercept)","logNt", coef_names[grep("meada",coef_names)])
clim_names <- coef_names[is.element(coef_names,cut_names)==F]
ridge_coefs <- ridge_fit$glmnet.fit$beta[,which(ridge_fit$lambda==ridge_fit$lambda.min)]
ridge_coefs <- ridge_coefs[is.element(names(ridge_coefs),clim_names)]
nopen_coefs <- coef(nopenalty_fit)
nopen_coefs <- nopen_coefs[is.element(names(nopen_coefs),clim_names)]
compare_coefs <- data.frame(model=c(rep("ridge",length(clim_names)),rep("nopenalty",length(clim_names))),
                            coefs = c(ridge_coefs,nopen_coefs))
rm(coef_names,cut_names,clim_names,ridge_coefs,nopen_coefs)

pdf('figures/shrinkage.pdf', height = 5.5, width = 3.5)

par(mfrow=c(2,1),tcl=-0.2,mgp=c(2,0.5,0),mar=c(3,4,1.5,1))

plot(log(ridge_fit$lambda),ridge_fit$cvm,
     xlab=expression(log(gamma)),ylab="Cross-validation MSE",type="l",lwd=2)
abline(v=log(ridge_fit$lambda.min),lty="dashed")
mtext("(A)",side=3,line=0.25,adj=0)

plot(as.numeric(compare_coefs[,1]),compare_coefs[,2],pch=16,
     xaxt="n",yaxt="n",xlab="",ylab="Coefficient",xlim=c(0.7,2.3))
axis(1,at=c(1,2),labels=c("No penalty","Ridge"))
axis(2,at=c(-0.2,0,0.2))
arrows(x0=rep(1,nrow(compare_coefs)/2),
       y0=compare_coefs[compare_coefs[,1]=="nopenalty",2],
       x1=rep(2,nrow(compare_coefs)/2),
       y1=compare_coefs[compare_coefs[,1]=="ridge",2],
       length=0)
abline(h=0,lty="dashed")
mtext("(B)",side=3,line=0.25,adj=0)

dev.off()


# plot observation vs predictions------------------------------------------

obs_pred <- read.csv("results/obs_vs_pred.csv",header=T)

year_key <- as.numeric(as.factor(obs_pred$year))
my_cols <- topo.colors(max(year_key),alpha=0.7)
my_symbols <- rep(c(1,2,3,4,5,6,15,16,17,18),2)

pdf("figures/ridge_obs_pred.pdf",height=3.25,width=7)

par(mfrow=c(1,2),tcl=-0.2,mgp=c(2,0.5,0),
    mar=c(2,2,1,1),oma=c(1,1,0,0),font.main=1,cex.main=1)

plot(obs_pred$pred_null,obs_pred$obs,
     xlim=c(-0.35,3),ylim=c(-0.35,3),xaxt="n",yaxt="n",
     ylab="",xlab="",pch=my_symbols[year_key],
     col=my_cols[year_key],main="Null model")
axis(1,at=c(0,1,2,3))
axis(2,at=c(0,1,2,3))
abline(0,1)

plot(obs_pred$pred_opt,obs_pred$obs,
     xlim=c(-0.35,3),ylim=c(-0.35,3),xaxt="n",yaxt="n",
     ylab="",xlab="",pch=my_symbols[year_key],
     col=my_cols[year_key],main="Ridge regression")
axis(1,at=c(0,1,2,3))
axis(2,at=c(0,1,2,3))
abline(0,1)

mtext(side=1,"Predicted",outer=T)
mtext(side=2,"Observed",outer=T)

dev.off()

# Resources-table ---------------------------------------------------
mytable <- tibble(
  Language = c("R", "R", "R", "R", "R", "Python"),
  Package = c("\\texttt{glmnet}", "\\texttt{glmmLasso}", "\\texttt{lmmen}", "\\texttt{climwin}", "\\texttt{loo}", "\\texttt{scikit-learn}"),
  `Statistical technique(s)` = c("regularization via Lasso, ridge regression, and elastic net; cross-validation",
                                 "Lasso for generalized linear mixed-effects models",
                                 "elastic net for generalized linear mixed-effects models",
                                 "moving window analysis for time-indexed covariates",
                                 "cross-validation and model scoring with MCMC; stratified blocking for validation sets",
                                 "machine learning; regularization via Lasso, ridge regression, and elastic net; cross-validation"),
  Citation = c("Friedman et al. (2010b)", "Groll (2017)", "Sidi (2017)", "Bailey and van de Pol (2016)", "Vehtari et al. (2018)", "Pedregosa et al. (2011)")) 

print(xtable(mytable,caption="Resources"),caption.placement="top",
      type="latex",file="results/resources.tex")

# Regularization concept figure ------------------------------------------------------

x = seq(-0.5,2,by=0.005)
loss = 2*abs(x-0.5)^2 + 0.4
P1 = abs(x)
P2 = x^2

pdf("./figures/new_regularization_plot.pdf", width = 11, 
    height = 4)

par(mfrow=c(1,2),bty="n",yaxs="i",xaxs="i",mar=c(3,4,3,2),mgp=c(2,1,0)); 

theColors=c("red","blue","purple","purple"); 

### Plot objective function and terms, LASSO  
matvec1 = cbind(loss,P1,loss+P1,loss+2*P1)
matplot(x,matvec1,ylim=c(0,2), xlab=expression(beta[1]),
        ylab='Objective function',lty=c(4,3,1,2), cex.lab=1.2,
        cex.axis=1,lwd=2,type='l', col=theColors,main="LASSO"); 
abline(h=0); 
abline(v=0);
legend('bottomright',
       c('Loss','Regulator','Loss + Regulator','Loss + 2*Regulator'),
       lwd=2,col=theColors,lty=c(4,3,1,2), bty="n",cex=1)

# add the minima to the objective function curves. 
j=which(loss==min(loss)); points(x[j],loss[j],pch=1,lwd=2,cex=1.5,col=theColors[1]); 
j=which(loss+P1==min(loss+P1)); points(x[j],loss[j]+P1[j],pch=16,cex=1.5,col=theColors[3]); 
j=which(loss+2*P1==min(loss+2*P1)); points(x[j],loss[j]+2*P1[j],pch=15,cex=1.5,col=theColors[3]); 

### Ditto, for Ridge Regression 
matvec2 = cbind(loss,P2,loss+P2,loss+2*P2)
matplot(x,matvec2,ylim=c(0,2),xlab=expression(beta[1]),ylab='Objective function',lty=c(4,3,1,2), cex.lab=1.2,cex.axis=1,lwd=2,type='l',col=theColors,
        main="Ridge Regression")
abline(h=0); abline(v=0); 
j=which(loss==min(loss)); points(x[j],loss[j],pch=1,cex=1.5,lwd=2,col=theColors[1]); 
j=which(loss+P2==min(loss+P2)); points(x[j],loss[j]+P2[j],pch=16,cex=1.5,col=theColors[3]); 
j=which(loss+2*P2==min(loss+2*P2)); points(x[j],loss[j]+2*P2[j],pch=15,cex=1.5,col=theColors[3]); 

dev.off()



